@@ -1,20 +1,9 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2000 Torben Weis <weis@kde.org>
-   Copyright (C) 2006 David Faure <faure@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2000 Torben Weis <weis@kde.org>
+    SPDX-FileCopyrightText: 2006 David Faure <faure@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #ifndef KMIMETYPETRADER_H
@@ -25,10 +14,11 @@ class KMimeTypeTraderPrivate;
 class KServiceOffer;
 typedef QList<KServiceOffer> KServiceOfferList;
 
+#if KSERVICE_ENABLE_DEPRECATED_SINCE(5, 82)
 /**
  * @class KMimeTypeTrader kmimetypetrader.h <KMimeTypeTrader>
  *
- * KDE's trader for services associated to a given mimetype.
+ * KDE's trader for services associated to a given MIME type.
  *
  * Example: say that you want to the list of all KParts components that can handle HTML.
  * Our code would look like:
@@ -45,23 +35,26 @@ typedef QList<KServiceOffer> KServiceOfferList;
  * Although if this is about loading that component you would use createPartInstanceFromQuery() directly.
  *
  * @see KServiceTypeTrader, KService
+ *
+ * @deprecated since 5.82. For querying applications use KApplicationTrader.
+ * For querying KParts use KParts::PartLoader.
+ * For querying plugins use KPluginLoader.
  */
 class KSERVICE_EXPORT KMimeTypeTrader
 {
 public:
-
     /**
      * Standard destructor
      */
     ~KMimeTypeTrader();
 
     /**
-     * This method returns a list of services which are associated with a given mimetype.
+     * This method returns a list of services which are associated with a given MIME type.
      *
      * Example usage:
-     * To get list of applications that can handle a given mimetype,
+     * To get list of applications that can handle a given MIME type,
      * set @p genericServiceType to "Application" (which is the default).
-     * To get list of embeddable components that can handle a given mimetype,
+     * To get list of embeddable components that can handle a given MIME type,
      * set @p genericServiceType to "KParts/ReadOnlyPart".
      *
      * The constraint parameter is used to limit the possible choices
@@ -77,7 +70,7 @@ public:
      * The keys used in the query (Type, ServiceTypes, Library) are all
      * fields found in the .desktop files.
      *
-     * @param mimeType A mime type like 'text/plain' or 'text/html'.
+     * @param mimeType a MIME type like 'text/plain' or 'text/html'
      * @param genericServiceType a basic service type, like 'KParts/ReadOnlyPart' or 'Application'
      * @param constraint  A constraint to limit the choices returned, QString() to
      *                    get all services that can handle the given @p mimetype
@@ -85,10 +78,13 @@ public:
      * @return A list of services that satisfy the query, sorted by preference
      * (preferred service first)
      * @see http://techbase.kde.org/Development/Tutorials/Services/Traders#The_KTrader_Query_Language
+     * @deprecated since 5.82. For querying applications use KApplicationTrader::query().
+     * For querying KParts use KParts::PartLoader::partsForMimeType().
+     * For querying plugins use KPluginLoader.
      */
-    KService::List query(const QString &mimeType,
-                         const QString &genericServiceType = QStringLiteral("Application"),
-                         const QString &constraint = QString()) const;
+    KSERVICE_DEPRECATED_VERSION(5, 82, "See API docs.")
+    KService::List
+    query(const QString &mimeType, const QString &genericServiceType = QStringLiteral("Application"), const QString &constraint = QString()) const;
 
     /**
      * Returns the preferred service for @p mimeType and @p genericServiceType
@@ -96,10 +92,13 @@ public:
      * This is almost like query().first(), except that it also checks
      * if the service is allowed as a preferred service (see KService::allowAsDefault).
      *
-     * @param mimeType the mime type (see query())
+     * @param mimeType the MIME type (see query())
      * @param genericServiceType the service type (see query())
-     * @return the preferred service, or @c nullptr if no service is available
+     * @return the preferred service, or @c nullptr if no service is available.
+     * @deprecated since 5.82. For querying applications use KApplicationTrader::preferredService(). For querying KParts use
+     * KParts::PartLoader::partsForMimeType().first().
      */
+    KSERVICE_DEPRECATED_VERSION(5, 82, "See API docs.")
     KService::Ptr preferredService(const QString &mimeType, const QString &genericServiceType = QStringLiteral("Application"));
 
     /**
@@ -114,7 +113,7 @@ public:
      * }
      * \endcode
      *
-     * @param mimeType the mimetype which this part is associated with
+     * @param mimeType the MIME type which this part is associated with
      * @param parentWidget the parent widget, will be set as the parent of the part's widget
      * @param parent the parent object for the part itself
      * @param constraint an optional constraint to pass to the trader
@@ -122,9 +121,14 @@ public:
      * @param error The string passed here will contain an error description.
      * @return A pointer to the newly created object or a null pointer if the
      *         factory was unable to create an object of the given type.
+     * @deprecated since 5.82. For KParts use KParts::PartLoader::createPartInstanceForMimeType().
+     * Otherwise use KPluginLoader.
      */
-    template <class T>
-    static T *createPartInstanceFromQuery(const QString &mimeType, QWidget *parentWidget = nullptr, QObject *parent = nullptr,
+    template<class T>
+    KSERVICE_DEPRECATED_VERSION(5, 82, "See API docs.")
+    static T *createPartInstanceFromQuery(const QString &mimeType,
+                                          QWidget *parentWidget = nullptr,
+                                          QObject *parent = nullptr,
                                           const QString &constraint = QString(),
                                           const QVariantList &args = QVariantList(),
                                           QString *error = nullptr)
@@ -146,20 +150,25 @@ public:
     }
 
     /**
-     * This can be used to create a service instance from a mime type query
+     * This can be used to create a service instance from a MIME type query.
      *
-     * @param mimeType A mime type like 'text/plain' or 'text/html'.
+     * @param mimeType a MIME type like 'text/plain' or 'text/html'
      * @param serviceType a basic service type
      * @param parent the parent object for the plugin itself
      * @param constraint  A constraint to limit the choices returned, QString() to
-     *                    get all services that can handle the given @p mimetype
+     *                    get all services that can handle the given @p mimeType
      * @param args A list of arguments passed to the service component
      * @param error The string passed here will contain an error description.
      * @return A pointer to the newly created object or a null pointer if the
      *         factory was unable to create an object of the given type.
+     * @deprecated since 5.82. For KParts use KParts::PartLoader::createPartInstanceForMimeType().
+     * Otherwise use KPluginLoader.
      */
-    template <class T>
-    static T *createInstanceFromQuery(const QString &mimeType, const QString &serviceType, QObject *parent = nullptr,
+    template<class T>
+    KSERVICE_DEPRECATED_VERSION(5, 82, "See API docs.")
+    static T *createInstanceFromQuery(const QString &mimeType,
+                                      const QString &serviceType,
+                                      QObject *parent = nullptr,
                                       const QString &constraint = QString(),
                                       const QVariantList &args = QVariantList(),
                                       QString *error = nullptr)
@@ -205,4 +214,6 @@ private:
     friend class KMimeTypeTraderSingleton;
 };
 
+#endif
+
 #endif /* KMIMETYPETRADER_H */
